@@ -16,6 +16,7 @@
 #include <limits.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <errno.h>
 
 #include <libfyaml.h>
 
@@ -29,8 +30,12 @@ void fy_emit_printf(struct fy_emitter *emit, enum fy_emitter_write_type type, co
 
 static inline bool fy_emit_is_json_mode(const struct fy_emitter *emit)
 {
-	enum fy_emitter_cfg_flags flags = emit->cfg.flags & FYECF_MODE(FYECF_MODE_MASK);
+	enum fy_emitter_cfg_flags flags;
 
+	if (emit->force_json)
+		return true;
+
+	flags = emit->cfg.flags & FYECF_MODE(FYECF_MODE_MASK);
 	return flags == FYECF_MODE_JSON || flags == FYECF_MODE_JSON_TP || flags == FYECF_MODE_JSON_ONELINE;
 }
 
@@ -45,7 +50,7 @@ static inline bool fy_emit_is_block_mode(const struct fy_emitter *emit)
 {
 	enum fy_emitter_cfg_flags flags = emit->cfg.flags & FYECF_MODE(FYECF_MODE_MASK);
 
-	return flags == FYECF_MODE_BLOCK || flags == FYECF_MODE_DEJSON;
+	return flags == FYECF_MODE_BLOCK || flags == FYECF_MODE_DEJSON || flags == FYECF_MODE_PRETTY;
 }
 
 static inline bool fy_emit_is_oneline(const struct fy_emitter *emit)
@@ -55,13 +60,20 @@ static inline bool fy_emit_is_oneline(const struct fy_emitter *emit)
 	return flags == FYECF_MODE_FLOW_ONELINE || flags == FYECF_MODE_JSON_ONELINE;
 }
 
-static inline bool fy_emit_is_pretty_mode(const struct fy_emitter *emit)
+static inline bool fy_emit_is_dejson_mode(const struct fy_emitter *emit)
 {
 	enum fy_emitter_cfg_flags flags = emit->cfg.flags & FYECF_MODE(FYECF_MODE_MASK);
 
 	return flags == FYECF_MODE_DEJSON;
 }
 
+static inline bool fy_emit_is_pretty_mode(const struct fy_emitter *emit)
+{
+	enum fy_emitter_cfg_flags flags = emit->cfg.flags & FYECF_MODE(FYECF_MODE_MASK);
+
+	return flags == FYECF_MODE_PRETTY;
+}
+
 static inline int fy_emit_indent(struct fy_emitter *emit)
 {
 	int indent;
@@ -141,7 +153,7 @@ static int fy_emit_node_check(struct fy_emitter *emit, struct fy_node *fyn)
 	if (!fyn)
 		return 0;
 
-	if (fy_emit_is_json_mode(emit)) {
+	if (fy_emit_is_json_mode(emit) && !emit->source_json) {
 		ret = fy_emit_node_check_json(emit, fyn);
 		if (ret)
 			return ret;
@@ -450,6 +462,34 @@ struct fy_atom *fy_emit_token_comment_handle(struct fy_emitter *emit, struct fy_
 	return handle && fy_atom_is_set(handle) ? handle : NULL;
 }
 
+void fy_emit_document_start_indicator(struct fy_emitter *emit)
+{
+	/* do not emit twice */
+	if (emit->flags & FYEF_HAD_DOCUMENT_START)
+		return;
+
+	/* do not try to emit if it's json mode */
+	if (fy_emit_is_json_mode(emit))
+		goto no_doc_emit;
+
+	/* output linebreak anyway */
+	if (emit->column)
+		fy_emit_putc(emit, fyewt_linebreak, '\n');
+
+	/* stripping doc indicators, do not emit */
+	if (emit->cfg.flags & FYECF_STRIP_DOC)
+		goto no_doc_emit;
+
+	/* ok, emit document start indicator */
+	fy_emit_puts(emit, fyewt_document_indicator, "---");
+	emit->flags &= ~FYEF_WHITESPACE;
+	emit->flags |= FYEF_HAD_DOCUMENT_START;
+	return;
+
+no_doc_emit:
+	emit->flags &= ~FYEF_HAD_DOCUMENT_START;
+}
+
 struct fy_token *fy_node_value_token(struct fy_node *fyn)
 {
 	struct fy_token *fyt;
@@ -617,6 +657,10 @@ void fy_emit_node_internal(struct fy_emitter *emit, struct fy_node *fyn, int fla
 
 	switch (type) {
 	case FYNT_SCALAR:
+		/* if we're pretty and root at column 0 (meaning it's a single scalar document) output --- */
+		if ((flags & DDNF_ROOT) && fy_emit_is_pretty_mode(emit) && !emit->column &&
+				!fy_emit_is_flow_mode(emit) && !(flags & DDNF_FLOW))
+			fy_emit_document_start_indicator(emit);
 		fy_emit_scalar(emit, fyn, flags, indent, is_key);
 		break;
 	case FYNT_SEQUENCE:
@@ -1216,10 +1260,8 @@ fy_emit_token_scalar_style(struct fy_emitter *emit, struct fy_token *fyt,
 	if (json && (style == FYNS_LITERAL || style == FYNS_FOLDED))
 		return FYNS_DOUBLE_QUOTED;
 
-	is_json_plain = false;
-
 	/* is this a plain json atom? */
-	is_json_plain = (json || fy_emit_is_pretty_mode(emit)) &&
+	is_json_plain = (json || emit->source_json || fy_emit_is_dejson_mode(emit)) &&
 			(!atom || atom->size0 ||
 			!fy_atom_strcmp(atom, "false") ||
 			!fy_atom_strcmp(atom, "true") ||
@@ -1237,10 +1279,7 @@ fy_emit_token_scalar_style(struct fy_emitter *emit, struct fy_token *fyt,
 	}
 
 	/* JSON NULL, but with plain style */
-	if (json && (!atom || atom->size0) && style == FYNS_PLAIN)
-		return FYNS_PLAIN;
-
-	if (json && style == FYNS_PLAIN && is_json_plain)
+	if (json && (style == FYNS_PLAIN || style == FYNS_ANY) && (!atom || (is_json_plain && !atom->size0)))
 		return FYNS_PLAIN;
 
 	if (json)
@@ -1269,11 +1308,29 @@ fy_emit_token_scalar_style(struct fy_emitter *emit, struct fy_token *fyt,
 		style = !(aflags & FYTTAF_EMPTY) ? FYNS_PLAIN : FYNS_DOUBLE_QUOTED;
 	}
 
+	/* try to pretify */
 	if (!flow && fy_emit_is_pretty_mode(emit) &&
 		(style == FYNS_ANY || style == FYNS_DOUBLE_QUOTED || style == FYNS_SINGLE_QUOTED)) {
 
-		if ((aflags & FYTTAF_CAN_BE_PLAIN) && (style != FYNS_DOUBLE_QUOTED || !is_json_plain))
+		/* any original style can be a plain, but contains linebreaks, do a literal */
+		if ((aflags & (FYTTAF_CAN_BE_PLAIN | FYTTAF_HAS_LB)) == (FYTTAF_CAN_BE_PLAIN | FYTTAF_HAS_LB)) {
+			style = FYNS_LITERAL;
+			goto out;
+		}
+
+		/* any style, can be just a plain, just make it so */
+		if (style == FYNS_ANY && (aflags & (FYTTAF_CAN_BE_PLAIN | FYTTAF_HAS_LB)) == FYTTAF_CAN_BE_PLAIN) {
 			style = FYNS_PLAIN;
+			goto out;
+		}
+
+	}
+
+	if (!flow && emit->source_json && fy_emit_is_dejson_mode(emit)) {
+		if (is_json_plain || (aflags & (FYTTAF_CAN_BE_PLAIN | FYTTAF_HAS_LB)) == FYTTAF_CAN_BE_PLAIN) {
+			style = FYNS_PLAIN;
+			goto out;
+		}
 	}
 
 out:
@@ -1349,6 +1406,7 @@ static void fy_emit_sequence_prolog(struct fy_emitter *emit, struct fy_emit_save
 {
 	bool json = fy_emit_is_json_mode(emit);
 	bool oneline = fy_emit_is_oneline(emit);
+	bool was_flow = sc->flow;
 
 	sc->old_indent = sc->indent;
 	if (!json) {
@@ -1373,8 +1431,10 @@ static void fy_emit_sequence_prolog(struct fy_emitter *emit, struct fy_emit_save
 		fy_emit_write_indicator(emit, di_left_bracket, sc->flags, sc->indent, fyewt_indicator);
 	}
 
-	if (!oneline)
-		sc->indent = fy_emit_increase_indent(emit, sc->flags, sc->indent);
+	if (!oneline) {
+		if (was_flow || (sc->flags & (DDNF_ROOT | DDNF_SEQ)))
+			sc->indent = fy_emit_increase_indent(emit, sc->flags, sc->indent);
+	}
 
 	sc->flags &= ~DDNF_ROOT;
 }
@@ -1448,7 +1508,7 @@ void fy_emit_sequence(struct fy_emitter *emit, struct fy_node *fyn, int flags, i
 		fyt_value = fy_node_value_token(fyni);
 
 		fy_emit_sequence_item_prolog(emit, sc, fyt_value);
-		fy_emit_node_internal(emit, fyni, sc->flags, sc->indent, false);
+		fy_emit_node_internal(emit, fyni, (sc->flags & ~DDNF_ROOT), sc->indent, false);
 		fy_emit_sequence_item_epilog(emit, sc, last, fyt_value);
 	}
 
@@ -1571,8 +1631,8 @@ void fy_emit_mapping(struct fy_emitter *emit, struct fy_node *fyn, int flags, in
 {
 	struct fy_node_pair *fynp, *fynpn, **fynpp = NULL;
 	struct fy_token *fyt_key, *fyt_value;
-	bool last, simple_key;
-	int aflags, i;
+	bool last, simple_key, used_malloc = false;
+	int aflags, i, count;
 	struct fy_emit_save_ctx sct, *sc = &sct;
 
 	memset(sc, 0, sizeof(*sc));
@@ -1586,11 +1646,39 @@ void fy_emit_mapping(struct fy_emitter *emit, struct fy_node *fyn, int flags, in
 
 	fy_emit_mapping_prolog(emit, sc);
 
-	if (!(emit->cfg.flags & FYECF_SORT_KEYS)) {
+	if (!(emit->cfg.flags & (FYECF_SORT_KEYS | FYECF_STRIP_EMPTY_KV))) {
 		fynp = fy_node_pair_list_head(&fyn->mapping);
 		fynpp = NULL;
 	} else {
-		fynpp = fy_node_mapping_sort_array(fyn, NULL, NULL, NULL);
+		count = fy_node_mapping_item_count(fyn);
+
+		/* heuristic, avoid allocation for small maps */
+		if (count > 64) {
+			fynpp = malloc((count + 1) * sizeof(*fynpp));
+			fyd_error_check(fyn->fyd, fynpp, err_out,
+					"malloc() failed");
+			used_malloc = true;
+		} else
+			fynpp = alloca((count + 1) * sizeof(*fynpp));
+
+		/* fill (removing empty KVs) */
+		i = 0;
+		for (fynp = fy_node_pair_list_head(&fyn->mapping); fynp;
+				fynp = fy_node_pair_next(&fyn->mapping, fynp)) {
+
+			/* strip key/value pair from the output if it's empty */
+			if ((emit->cfg.flags & FYECF_STRIP_EMPTY_KV) && fy_node_is_empty(fynp->value))
+				continue;
+
+			fynpp[i++] = fynp;
+		}
+		count = i;
+		fynpp[count] = NULL;
+
+		/* sort the keys */
+		if (emit->cfg.flags & FYECF_SORT_KEYS)
+			fy_node_mapping_perform_sort(fyn, NULL, NULL, fynpp, count);
+
 		i = 0;
 		fynp = fynpp[i];
 	}
@@ -1630,17 +1718,17 @@ void fy_emit_mapping(struct fy_emitter *emit, struct fy_node *fyn, int flags, in
 
 		fy_emit_mapping_key_prolog(emit, sc, fyt_key, simple_key);
 		if (fynp->key)
-			fy_emit_node_internal(emit, fynp->key, sc->flags, sc->indent, true);
+			fy_emit_node_internal(emit, fynp->key, (sc->flags & ~DDNF_ROOT), sc->indent, true);
 		fy_emit_mapping_key_epilog(emit, sc, fyt_key);
 
 		fy_emit_mapping_value_prolog(emit, sc, fyt_value);
 		if (fynp->value)
-			fy_emit_node_internal(emit, fynp->value, sc->flags, sc->indent, false);
+			fy_emit_node_internal(emit, fynp->value, (sc->flags & ~DDNF_ROOT), sc->indent, false);
 		fy_emit_mapping_value_epilog(emit, sc, last, fyt_value);
 	}
 
-	if (fynpp)
-		fy_node_mapping_sort_release_array(fyn, fynpp);
+	if (fynpp && used_malloc)
+		free(fynpp);
 
 	fy_emit_mapping_epilog(emit, sc);
 
@@ -1736,16 +1824,9 @@ int fy_emit_common_document_start(struct fy_emitter *emit,
 	           !(emit->flags & FYEF_HAD_DOCUMENT_END))
 		dsm = true;
 
-	if (!fy_emit_is_json_mode(emit) && dsm) {
-		if (emit->column)
-			fy_emit_putc(emit, fyewt_linebreak, '\n');
-		if (!(emit->cfg.flags & FYECF_STRIP_DOC)) {
-			fy_emit_puts(emit, fyewt_document_indicator, "---");
-			emit->flags &= ~FYEF_WHITESPACE;
-			emit->flags |= FYEF_HAD_DOCUMENT_START;
-		}
-	} else
-		emit->flags &= ~FYEF_HAD_DOCUMENT_START;
+	/* output document start indicator if we should */
+	if (dsm)
+		fy_emit_document_start_indicator(emit);
 
 	/* clear that in any case */
 	emit->flags &= ~FYEF_HAD_DOCUMENT_END;
@@ -1776,19 +1857,23 @@ int fy_emit_document_start(struct fy_emitter *emit, struct fy_document *fyd,
 	return 0;
 }
 
-int fy_emit_common_document_end(struct fy_emitter *emit)
+int fy_emit_common_document_end(struct fy_emitter *emit, bool override_state, bool implicit_override)
 {
 	const struct fy_document_state *fyds;
 	enum fy_emitter_cfg_flags flags = emit->cfg.flags;
 	enum fy_emitter_cfg_flags dem_flags = flags & FYECF_DOC_END_MARK(FYECF_DOC_END_MARK_MASK);
-	bool dem;
+	bool implicit, dem;
 
 	if (!emit || !emit->fyds)
 		return -1;
 
 	fyds = emit->fyds;
 
-	dem = ((dem_flags == FYECF_DOC_END_MARK_AUTO && !fyds->end_implicit) ||
+	implicit = fyds->end_implicit;
+	if (override_state)
+		implicit = implicit_override;
+
+	dem = ((dem_flags == FYECF_DOC_END_MARK_AUTO && !implicit) ||
 		dem_flags == FYECF_DOC_END_MARK_ON) &&
 	       !(emit->cfg.flags & FYECF_STRIP_DOC);
 
@@ -1831,7 +1916,7 @@ int fy_emit_document_end(struct fy_emitter *emit)
 {
 	int ret;
 
-	ret = fy_emit_common_document_end(emit);
+	ret = fy_emit_common_document_end(emit, false, false);
 	if (ret)
 		return ret;
 
@@ -1946,6 +2031,18 @@ int fy_emit_setup(struct fy_emitter *emit, const struct fy_emitter_cfg *cfg)
 	fy_eventp_list_init(&emit->recycled_eventp);
 	fy_token_list_init(&emit->recycled_token);
 
+	/* suppress recycling if we must */
+	emit->suppress_recycling_force = getenv("FY_VALGRIND") && !getenv("FY_VALGRIND_RECYCLING");
+	emit->suppress_recycling = emit->suppress_recycling_force;
+
+	if (!emit->suppress_recycling) {
+		emit->recycled_eventp_list = &emit->recycled_eventp;
+		emit->recycled_token_list = &emit->recycled_token;
+	} else {
+		emit->recycled_eventp_list = NULL;
+		emit->recycled_token_list = NULL;
+	}
+
 	fy_emit_reset(emit, false);
 
 	return 0;
@@ -1956,10 +2053,15 @@ void fy_emit_cleanup(struct fy_emitter *emit)
 	struct fy_eventp *fyep;
 	struct fy_token *fyt;
 
+	/* call the finalizer if it exists */
+	if (emit->finalizer)
+		emit->finalizer(emit);
+
 	while ((fyt = fy_token_list_pop(&emit->recycled_token)) != NULL)
 		fy_token_free(fyt);
 
-	fy_emit_eventp_vacuum(emit);
+	while ((fyep = fy_eventp_list_pop(&emit->recycled_eventp)) != NULL)
+		fy_eventp_free(fyep);
 
 	if (!emit->fyd && emit->fyds)
 		fy_document_state_unref(emit->fyds);
@@ -1978,6 +2080,13 @@ void fy_emit_cleanup(struct fy_emitter *emit)
 	fy_diag_unref(emit->diag);
 }
 
+int fy_emit_node_no_check(struct fy_emitter *emit, struct fy_node *fyn)
+{
+	if (fyn)
+		fy_emit_node_internal(emit, fyn, DDNF_ROOT, -1, false);
+	return 0;
+}
+
 int fy_emit_node(struct fy_emitter *emit, struct fy_node *fyn)
 {
 	int ret;
@@ -1986,22 +2095,14 @@ int fy_emit_node(struct fy_emitter *emit, struct fy_node *fyn)
 	if (ret)
 		return ret;
 
-	if (fyn)
-		fy_emit_node_internal(emit, fyn, DDNF_ROOT, -1, false);
-	return 0;
+	return fy_emit_node_no_check(emit, fyn);
 }
 
-int fy_emit_root_node(struct fy_emitter *emit, struct fy_node *fyn)
+int fy_emit_root_node_no_check(struct fy_emitter *emit, struct fy_node *fyn)
 {
-	int ret;
-
 	if (!emit || !fyn)
 		return -1;
 
-	ret = fy_emit_node_check(emit, fyn);
-	if (ret)
-		return ret;
-
 	/* top comment first */
 	fy_emit_node_comment(emit, fyn, DDNF_ROOT, -1, fycp_top);
 
@@ -2016,7 +2117,32 @@ int fy_emit_root_node(struct fy_emitter *emit, struct fy_node *fyn)
 	return 0;
 }
 
-int fy_emit_document(struct fy_emitter *emit, struct fy_document *fyd)
+int fy_emit_root_node(struct fy_emitter *emit, struct fy_node *fyn)
+{
+	int ret;
+
+	if (!emit || !fyn)
+		return -1;
+
+	ret = fy_emit_node_check(emit, fyn);
+	if (ret)
+		return ret;
+
+	return fy_emit_root_node_no_check(emit, fyn);
+}
+
+void fy_emit_prepare_document_state(struct fy_emitter *emit, struct fy_document_state *fyds)
+{
+	if (!emit || !fyds)
+		return;
+
+	/* if the original document was JSON and the mode is ORIGINAL turn on JSON mode */
+	emit->source_json = fyds && fyds->json_mode;
+	emit->force_json = (emit->cfg.flags & FYECF_MODE(FYECF_MODE_MASK)) == FYECF_MODE_ORIGINAL &&
+			   emit->source_json;
+}
+
+int fy_emit_document_no_check(struct fy_emitter *emit, struct fy_document *fyd)
 {
 	int rc;
 
@@ -2024,7 +2150,7 @@ int fy_emit_document(struct fy_emitter *emit, struct fy_document *fyd)
 	if (rc)
 		return rc;
 
-	rc = fy_emit_root_node(emit, fyd->root);
+	rc = fy_emit_root_node_no_check(emit, fyd->root);
 	if (rc)
 		return rc;
 
@@ -2033,6 +2159,26 @@ int fy_emit_document(struct fy_emitter *emit, struct fy_document *fyd)
 	return rc;
 }
 
+int fy_emit_document(struct fy_emitter *emit, struct fy_document *fyd)
+{
+	int ret;
+
+	if (!emit)
+		return -1;
+
+	if (fyd) {
+		fy_emit_prepare_document_state(emit, fyd->fyds);
+
+		if (fyd->root) {
+			ret = fy_emit_node_check(emit, fyd->root);
+			if (ret)
+				return ret;
+		}
+	}
+
+	return fy_emit_document_no_check(emit, fyd);
+}
+
 struct fy_emitter *fy_emitter_create(const struct fy_emitter_cfg *cfg)
 {
 	struct fy_emitter *emit;
@@ -2100,26 +2246,41 @@ int fy_emitter_set_diag(struct fy_emitter *emit, struct fy_diag *diag)
 	return 0;
 }
 
+void fy_emitter_set_finalizer(struct fy_emitter *emit,
+		void (*finalizer)(struct fy_emitter *emit))
+{
+	if (!emit)
+		return;
+	emit->finalizer = finalizer;
+}
+
 struct fy_emit_buffer_state {
+	char **bufp;
+	size_t *sizep;
 	char *buf;
-	int size;
-	int pos;
-	int need;
-	bool grow;
+	size_t size;
+	size_t pos;
+	size_t need;
+	bool allocate_buffer;
 };
 
-static int do_buffer_output(struct fy_emitter *emit, enum fy_emitter_write_type type, const char *str, int len, void *userdata)
+static int do_buffer_output(struct fy_emitter *emit, enum fy_emitter_write_type type, const char *str, int leni, void *userdata)
 {
 	struct fy_emit_buffer_state *state = emit->cfg.userdata;
-	int left;
-	int pagesize = 0;
-	int size;
+	size_t left, pagesize, size, len;
 	char *bufnew;
 
+	/* convert to unsigned and use that */
+	len = (size_t)leni;
+
+	/* no funky business */
+	if (len < 0)
+		return -1;
+
 	state->need += len;
 	left = state->size - state->pos;
 	if (left < len) {
-		if (!state->grow)
+		if (!state->allocate_buffer)
 			return 0;
 
 		pagesize = sysconf(_SC_PAGESIZE);
@@ -2144,64 +2305,162 @@ static int do_buffer_output(struct fy_emitter *emit, enum fy_emitter_write_type
 	return len;
 }
 
-static int fy_emit_str_internal(struct fy_document *fyd,
-				enum fy_emitter_cfg_flags flags,
-				struct fy_node *fyn, char **bufp, int *sizep,
-				bool grow)
+static void
+fy_emitter_str_finalizer(struct fy_emitter *emit)
 {
-	struct fy_emitter emit_state, *emit = &emit_state;
+	struct fy_emit_buffer_state *state;
+
+	if (!emit || !(state = emit->cfg.userdata))
+		return;
+
+	/* if the buffer is allowed to allocate_buffer... */
+	if (state->allocate_buffer && state->buf)
+		free(state->buf);
+	free(state);
+
+	emit->cfg.userdata = NULL;
+}
+
+static struct fy_emitter *
+fy_emitter_create_str_internal(enum fy_emitter_cfg_flags flags, char **bufp, size_t *sizep, bool allocate_buffer)
+{
+	struct fy_emitter *emit;
 	struct fy_emitter_cfg emit_cfg;
-	struct fy_emit_buffer_state state;
-	int rc = -1;
+	struct fy_emit_buffer_state *state;
+
+	state = malloc(sizeof(*state));
+	if (!state)
+		return NULL;
+
+	/* if any of these NULL, it's a allocation case */
+	if ((!bufp || !sizep) && !allocate_buffer)
+		return NULL;
+
+	if (bufp && sizep) {
+		state->bufp = bufp;
+		state->buf = *bufp;
+		state->sizep = sizep;
+		state->size = *sizep;
+	} else {
+		state->bufp = NULL;
+		state->buf = NULL;
+		state->sizep = NULL;
+		state->size = 0;
+	}
+	state->pos = 0;
+	state->need = 0;
+	state->allocate_buffer = allocate_buffer;
 
 	memset(&emit_cfg, 0, sizeof(emit_cfg));
-	memset(&state, 0, sizeof(state));
-
 	emit_cfg.output = do_buffer_output;
-	emit_cfg.userdata = &state;
+	emit_cfg.userdata = state;
 	emit_cfg.flags = flags;
-	state.buf = *bufp;
-	state.size = *sizep;
-	state.grow = grow;
 
-	fy_emit_setup(emit, &emit_cfg);
-	rc = fyd ? fy_emit_document(emit, fyd) : fy_emit_node(emit, fyn);
-	fy_emit_cleanup(emit);
+	emit = fy_emitter_create(&emit_cfg);
+	if (!emit)
+		goto err_out;
+
+	/* set finalizer to cleanup */
+	fy_emitter_set_finalizer(emit, fy_emitter_str_finalizer);
+
+	return emit;
+
+err_out:
+	if (state)
+		free(state);
+	return NULL;
+}
+
+static int
+fy_emitter_collect_str_internal(struct fy_emitter *emit, char **bufp, size_t *sizep)
+{
+	struct fy_emit_buffer_state *state;
+	char *buf;
+	int rc;
+
+	state = emit->cfg.userdata;
+	assert(state);
+
+	/* if NULL, then use the values stored on the state */
+	if (!bufp)
+		bufp = state->bufp;
+	if (!sizep)
+		sizep = state->sizep;
+
+	/* terminating zero */
+	rc = do_buffer_output(emit, fyewt_terminating_zero, "\0", 1, state);
+	if (rc != 1)
+		goto err_out;
+
+	state->size = state->need;
+
+	if (state->allocate_buffer) {
+		/* resize */
+		buf = realloc(state->buf, state->size);
+		/* very likely since we shrink the buffer, but make sure we don't error out */
+		if (buf)
+			state->buf = buf;
+	}
+
+	/* retreive the buffer and size */
+	*sizep = state->size;
+	*bufp = state->buf;
+
+	/* reset the buffer, ownership now to the caller */
+	state->buf = NULL;
+	state->size = 0;
+	state->pos = 0;
+	state->bufp = NULL;
+	state->sizep = NULL;
+
+	return 0;
+
+err_out:
+	*bufp = NULL;
+	*sizep = 0;
+	return -1;
+}
+
+static int fy_emit_str_internal(struct fy_document *fyd,
+				enum fy_emitter_cfg_flags flags,
+				struct fy_node *fyn, char **bufp, size_t *sizep,
+				bool allocate_buffer)
+{
+	struct fy_emitter *emit = NULL;
+	int rc = -1;
+
+	emit = fy_emitter_create_str_internal(flags, bufp, sizep, allocate_buffer);
+	if (!emit)
+		goto out_err;
+
+	if (fyd) {
+		fy_emit_prepare_document_state(emit, fyd->fyds);
+		rc = 0;
+		if (fyd->root)
+			rc = fy_emit_node_check(emit, fyd->root);
+		if (!rc)
+			rc = fy_emit_document_no_check(emit, fyd);
+	} else {
+		rc = fy_emit_node_check(emit, fyn);
+		if (!rc)
+			rc = fy_emit_node_no_check(emit, fyn);
+	}
 
 	if (rc)
 		goto out_err;
 
-	/* terminating zero */
-	rc = do_buffer_output(emit, fyewt_terminating_zero, "\0", 1, emit->cfg.userdata);
-
-	if (rc != 1) {
-		rc = -1;
+	rc = fy_emitter_collect_str_internal(emit, NULL, NULL);
+	if (rc)
 		goto out_err;
-	}
 
-	*sizep = state.need;
-
-	if (!grow)
-		return 0;
-
-	*bufp = realloc(state.buf, *sizep);
-	if (!*bufp) {
-		rc = -1;
-		goto out_err;
-	}
-
-	return 0;
+	/* OK, all done */
 
 out_err:
-	if (grow && state.buf)
-		free(state.buf);
-	*bufp = NULL;
-	*sizep = 0;
-
+	fy_emitter_destroy(emit);
 	return rc;
 }
 
-int fy_emit_document_to_buffer(struct fy_document *fyd, enum fy_emitter_cfg_flags flags, char *buf, int size)
+int fy_emit_document_to_buffer(struct fy_document *fyd, enum fy_emitter_cfg_flags flags, char *buf, size_t size)
 {
 	int rc;
 
@@ -2213,18 +2472,77 @@ int fy_emit_document_to_buffer(struct fy_document *fyd, enum fy_emitter_cfg_flag
 
 char *fy_emit_document_to_string(struct fy_document *fyd, enum fy_emitter_cfg_flags flags)
 {
-	char *buf = NULL;
-	int rc, size = 0;
+	char *buf;
+	size_t size;
+	int rc;
 
+	buf = NULL;
+	size = 0;
 	rc = fy_emit_str_internal(fyd, flags, NULL, &buf, &size, true);
 	if (rc != 0)
 		return NULL;
 	return buf;
 }
 
-static int do_file_output(struct fy_emitter *emit, enum fy_emitter_write_type type, const char *str, int len, void *userdata)
+struct fy_emitter *
+fy_emit_to_buffer(enum fy_emitter_cfg_flags flags, char *buf, size_t size)
+{
+	if (!buf)
+		return NULL;
+
+	return fy_emitter_create_str_internal(flags, &buf, &size, false);
+}
+
+char *
+fy_emit_to_buffer_collect(struct fy_emitter *emit, size_t *sizep)
+{
+	int rc;
+	char *buf;
+
+	if (!emit || !sizep)
+		return NULL;
+
+	rc = fy_emitter_collect_str_internal(emit, &buf, sizep);
+	if (rc) {
+		*sizep = 0;
+		return NULL;
+	}
+	return buf;
+}
+
+struct fy_emitter *
+fy_emit_to_string(enum fy_emitter_cfg_flags flags)
+{
+	return fy_emitter_create_str_internal(flags, NULL, NULL, true);
+}
+
+char *
+fy_emit_to_string_collect(struct fy_emitter *emit, size_t *sizep)
+{
+	int rc;
+	char *buf;
+
+	if (!emit || !sizep)
+		return NULL;
+
+	rc = fy_emitter_collect_str_internal(emit, &buf, sizep);
+	if (rc) {
+		*sizep = 0;
+		return NULL;
+	}
+	return buf;
+}
+
+static int do_file_output(struct fy_emitter *emit, enum fy_emitter_write_type type, const char *str, int leni, void *userdata)
 {
 	FILE *fp = userdata;
+	size_t len;
+
+	len = (size_t)leni;
+
+	/* no funky stuff */
+	if (len < 0)
+		return -1;
 
 	return fwrite(str, 1, len, fp);
 }
@@ -2244,7 +2562,15 @@ int fy_emit_document_to_fp(struct fy_document *fyd, enum fy_emitter_cfg_flags fl
 	emit_cfg.userdata = fp;
 	emit_cfg.flags = flags;
 	fy_emit_setup(emit, &emit_cfg);
-	rc = fy_emit_document(emit, fyd);
+
+	fy_emit_prepare_document_state(emit, fyd->fyds);
+
+	rc = 0;
+	if (fyd->root)
+		rc = fy_emit_node_check(emit, fyd->root);
+
+	rc = fy_emit_document_no_check(emit, fyd);
+
 	fy_emit_cleanup(emit);
 
 	return rc ? rc : 0;
@@ -2269,7 +2595,75 @@ int fy_emit_document_to_file(struct fy_document *fyd,
 	return rc ? rc : 0;
 }
 
-int fy_emit_node_to_buffer(struct fy_node *fyn, enum fy_emitter_cfg_flags flags, char *buf, int size)
+static int do_fd_output(struct fy_emitter *emit, enum fy_emitter_write_type type, const char *str, int leni, void *userdata)
+{
+	size_t len;
+	int fd;
+	ssize_t wrn;
+	int total;
+
+	len = (size_t)leni;
+
+	/* no funky stuff */
+	if (len < 0)
+		return -1;
+
+	/* get the file descriptor */
+	fd = (int)(uintptr_t)userdata;
+	if (fd < 0)
+		return -1;
+
+	/* loop output to fd */
+	total = 0;
+	while (len > 0) {
+
+		do {
+			wrn = write(fd, str, len);
+		} while (wrn == -1 && errno == EAGAIN);
+
+		if (wrn == -1)
+			return -1;
+
+		if (wrn == 0)
+			return total;
+
+		len -= wrn;
+		str += wrn;
+		total += wrn;
+	}
+
+	return total;
+}
+
+int fy_emit_document_to_fd(struct fy_document *fyd, enum fy_emitter_cfg_flags flags, int fd)
+{
+	struct fy_emitter emit_state, *emit = &emit_state;
+	struct fy_emitter_cfg emit_cfg;
+	int rc;
+
+	if (fd < 0)
+		return -1;
+
+	memset(&emit_cfg, 0, sizeof(emit_cfg));
+	emit_cfg.output = do_fd_output;
+	emit_cfg.userdata = (void *)(uintptr_t)fd;
+	emit_cfg.flags = flags;
+	fy_emit_setup(emit, &emit_cfg);
+
+	fy_emit_prepare_document_state(emit, fyd->fyds);
+
+	rc = 0;
+	if (fyd->root)
+		rc = fy_emit_node_check(emit, fyd->root);
+
+	rc = fy_emit_document_no_check(emit, fyd);
+
+	fy_emit_cleanup(emit);
+
+	return rc ? rc : 0;
+}
+
+int fy_emit_node_to_buffer(struct fy_node *fyn, enum fy_emitter_cfg_flags flags, char *buf, size_t size)
 {
 	int rc;
 
@@ -2281,9 +2675,12 @@ int fy_emit_node_to_buffer(struct fy_node *fyn, enum fy_emitter_cfg_flags flags,
 
 char *fy_emit_node_to_string(struct fy_node *fyn, enum fy_emitter_cfg_flags flags)
 {
-	char *buf = NULL;
-	int rc, size = 0;
+	char *buf;
+	size_t size;
+	int rc;
 
+	buf = NULL;
+	size = 0;
 	rc = fy_emit_str_internal(NULL, flags, fyn, &buf, &size, true);
 	if (rc != 0)
 		return NULL;
@@ -2498,6 +2895,10 @@ static int fy_emit_streaming_node(struct fy_emitter *emit, struct fy_eventp *fye
 		break;
 
 	case FYET_SCALAR:
+		/* if we're pretty and at column 0 (meaning it's a single scalar document) output --- */
+		if ((emit->s_flags & DDNF_ROOT) && fy_emit_is_pretty_mode(emit) && !emit->column &&
+				!fy_emit_is_flow_mode(emit) && !(emit->s_flags & DDNF_FLOW))
+			fy_emit_document_start_indicator(emit);
 		fy_emit_common_node_preamble(emit, fye->scalar.anchor, fye->scalar.tag, emit->s_flags, emit->s_indent);
 		style = fye->scalar.value ?
 				fy_node_style_from_scalar_style(fye->scalar.value->scalar.style) :
@@ -2520,7 +2921,7 @@ static int fy_emit_streaming_node(struct fy_emitter *emit, struct fy_eventp *fye
 
 		/* create new context */
 		memset(sc, 0, sizeof(*sc));
-		sc->flags = DDNF_SEQ | (emit->s_flags & DDNF_ROOT);
+		sc->flags = emit->s_flags & (DDNF_ROOT | DDNF_SEQ | DDNF_MAP);
 		sc->indent = emit->s_indent;
 		sc->empty = fy_emit_streaming_sequence_empty(emit);
 		sc->flow_token = fye->sequence_start.sequence_start &&
@@ -2533,6 +2934,8 @@ static int fy_emit_streaming_node(struct fy_emitter *emit, struct fy_eventp *fye
 		sc->s_indent = s_indent;
 
 		fy_emit_sequence_prolog(emit, sc);
+		sc->flags &= ~DDNF_MAP;
+		sc->flags |= DDNF_SEQ;
 
 		emit->s_flags = sc->flags;
 		emit->s_indent = sc->indent;
@@ -2553,7 +2956,7 @@ static int fy_emit_streaming_node(struct fy_emitter *emit, struct fy_eventp *fye
 
 		/* create new context */
 		memset(sc, 0, sizeof(*sc));
-		sc->flags = DDNF_MAP | (emit->s_flags & DDNF_ROOT);
+		sc->flags = emit->s_flags & (DDNF_ROOT | DDNF_SEQ | DDNF_MAP);
 		sc->indent = emit->s_indent;
 		sc->empty = fy_emit_streaming_mapping_empty(emit);
 		sc->flow_token = fye->mapping_start.mapping_start &&
@@ -2566,6 +2969,8 @@ static int fy_emit_streaming_node(struct fy_emitter *emit, struct fy_eventp *fye
 		sc->s_indent = s_indent;
 
 		fy_emit_mapping_prolog(emit, sc);
+		sc->flags &= ~DDNF_SEQ;
+		sc->flags |= DDNF_MAP;
 
 		emit->s_flags = sc->flags;
 		emit->s_indent = sc->indent;
@@ -2617,6 +3022,9 @@ static int fy_emit_handle_document_start(struct fy_emitter *emit, struct fy_even
 	fyds = fye->document_start.document_state;
 	fye->document_start.document_state = NULL;
 
+	/* prepare (i.e. adapt to the document state) */
+	fy_emit_prepare_document_state(emit, fyds);
+
 	fy_emit_common_document_start(emit, fyds, false);
 
 	fy_emit_goto_state(emit, FYES_DOCUMENT_CONTENT);
@@ -2624,17 +3032,6 @@ static int fy_emit_handle_document_start(struct fy_emitter *emit, struct fy_even
 	return 0;
 }
 
-static int fy_emit_handle_document_content(struct fy_emitter *emit, struct fy_eventp *fyep)
-{
-	int ret;
-
-	ret = fy_emit_push_state(emit, FYES_DOCUMENT_END);
-	if (ret)
-		return ret;
-
-	return fy_emit_streaming_node(emit, fyep, DDNF_ROOT);
-}
-
 static int fy_emit_handle_document_end(struct fy_emitter *emit, struct fy_eventp *fyep)
 {
 	struct fy_document_state *fyds;
@@ -2648,7 +3045,7 @@ static int fy_emit_handle_document_end(struct fy_emitter *emit, struct fy_eventp
 
 	fyds = emit->fyds;
 
-	ret = fy_emit_common_document_end(emit);
+	ret = fy_emit_common_document_end(emit, true, fye->document_end.implicit);
 	if (ret)
 		return ret;
 
@@ -2659,6 +3056,22 @@ static int fy_emit_handle_document_end(struct fy_emitter *emit, struct fy_eventp
 	return 0;
 }
 
+static int fy_emit_handle_document_content(struct fy_emitter *emit, struct fy_eventp *fyep)
+{
+	struct fy_event *fye = &fyep->e;
+	int ret;
+
+	/* empty document? */
+	if (fye->type == FYET_DOCUMENT_END)
+		return fy_emit_handle_document_end(emit, fyep);
+
+	ret = fy_emit_push_state(emit, FYES_DOCUMENT_END);
+	if (ret)
+		return ret;
+
+	return fy_emit_streaming_node(emit, fyep, DDNF_ROOT);
+}
+
 static int fy_emit_handle_sequence_item(struct fy_emitter *emit, struct fy_eventp *fyep, bool first)
 {
 	struct fy_event *fye = &fyep->e;
@@ -2666,7 +3079,7 @@ static int fy_emit_handle_sequence_item(struct fy_emitter *emit, struct fy_event
 	struct fy_token *fyt_item = NULL;
 	int ret;
 
-	fy_token_unref_rl(&emit->recycled_token, sc->fyt_last_value);
+	fy_token_unref_rl(emit->recycled_token_list, sc->fyt_last_value);
 	sc->fyt_last_value = NULL;
 
 	switch (fye->type) {
@@ -2747,9 +3160,9 @@ static int fy_emit_handle_mapping_key(struct fy_emitter *emit, struct fy_eventp
 	int ret, aflags;
 	bool simple_key;
 
-	fy_token_unref_rl(&emit->recycled_token, sc->fyt_last_key);
+	fy_token_unref_rl(emit->recycled_token_list, sc->fyt_last_key);
 	sc->fyt_last_key = NULL;
-	fy_token_unref_rl(&emit->recycled_token, sc->fyt_last_value);
+	fy_token_unref_rl(emit->recycled_token_list, sc->fyt_last_value);
 	sc->fyt_last_value = NULL;
 
 	simple_key = false;
@@ -3098,7 +3511,7 @@ int fy_emitter_default_output(struct fy_emitter *fye, enum fy_emitter_write_type
 
 int fy_document_default_emit_to_fp(struct fy_document *fyd, FILE *fp)
 {
-	struct fy_emitter fye_local, *fye = &fye_local;
+	struct fy_emitter emit_local, *emit = &emit_local;
 	struct fy_emitter_cfg ecfg_local, *ecfg = &ecfg_local;
 	struct fy_emitter_default_output_data d_local, *d = &d_local;
 	int rc;
@@ -3112,20 +3525,26 @@ int fy_document_default_emit_to_fp(struct fy_document *fyd, FILE *fp)
 	ecfg->diag = fyd->diag;
 	ecfg->userdata = d;
 
-	rc = fy_emit_setup(fye, ecfg);
+	rc = fy_emit_setup(emit, ecfg);
 	if (rc)
 		goto err_setup;
 
-	rc = fy_emit_document(fye, fyd);
+	fy_emit_prepare_document_state(emit, fyd->fyds);
+
+	rc = 0;
+	if (fyd->root)
+		rc = fy_emit_node_check(emit, fyd->root);
+
+	rc = fy_emit_document_no_check(emit, fyd);
 	if (rc)
 		goto err_emit;
 
-	fy_emit_cleanup(fye);
+	fy_emit_cleanup(emit);
 
 	return 0;
 
 err_emit:
-	fy_emit_cleanup(fye);
+	fy_emit_cleanup(emit);
 err_setup:
 	return -1;
 }
