@@ -38,6 +38,7 @@ MRPT_TEST(yaml, assignments)
 	p["enabled"] = true;
 	p["visible"] = false;
 	p["hidden"] = "true";
+	p["val_of_size_t"] = size_t(10);
 	auto nHidden1 = p["hidden"];
 
 	EXPECT_FALSE(p.empty());
@@ -55,6 +56,8 @@ MRPT_TEST(yaml, assignments)
 	EXPECT_FALSE(p["visible"]);
 	EXPECT_TRUE(p["hidden"]);
 
+	EXPECT_EQ(p["val_of_size_t"].as<size_t>(), size_t(10));
+
 	EXPECT_EQ(p["K"].scalarType(), typeid(double));
 	EXPECT_EQ(p["K"].as<double>(), 2.0);
 	EXPECT_EQ(p["K"].as<float>(), 2.0f);
