@@ -2,7 +2,7 @@
    |                     Mobile Robot Programming Toolkit (MRPT)            |
    |                          https://www.mrpt.org/                         |
    |                                                                        |
-   | Copyright (c) 2005-2021, Individual contributors, see AUTHORS file     |
+   | Copyright (c) 2005-2022, Individual contributors, see AUTHORS file     |
    | See: https://www.mrpt.org/Authors - All rights reserved.               |
    | Released under BSD License. See: https://www.mrpt.org/License          |
    +------------------------------------------------------------------------+ */
@@ -341,9 +341,9 @@ void WxSubsystem::CWXMainFrame::OnTimerProcessRequests(wxTimerEvent& event)
 							// img->GetHeight());
 						}
 						delete img;
-						wnd->m_image->Refresh(false);  // false: Do NOT erase
-						// background: avoid
-						// flickering
+						// false: Do NOT erase background: avoid flickering
+						wnd->m_image->Refresh(false);
+						wnd->m_image->Update();
 					}
 					break;
 				// Set position
