@@ -9,6 +9,7 @@
 
 #include "opengl-precomp.h"	 // Precompiled header
 //
+#include <mrpt/containers/yaml.h>
 #include <mrpt/core/round.h>  // round()
 #include <mrpt/math/ops_containers.h>
 #include <mrpt/opengl/CPointCloud.h>
@@ -443,3 +444,10 @@ void CPointCloud::setAllPoints(const std::vector<mrpt::math::TPoint3D>& pts)
 	markAllPointsAsNew();
 	CRenderizable::notifyChange();
 }
+
+void CPointCloud::toYAMLMap(mrpt::containers::yaml& propertiesMap) const
+{
+	CRenderizable::toYAMLMap(propertiesMap);
+	propertiesMap["point_count"] = m_points.size();
+	propertiesMap["bounding_box"] = this->getBoundingBox().asString();
+}
