@@ -9,6 +9,7 @@
 
 #include "opengl-precomp.h"	 // Precompiled header
 //
+#include <mrpt/containers/yaml.h>
 #include <mrpt/core/round.h>  // round()
 #include <mrpt/math/ops_containers.h>  // for << ops
 #include <mrpt/opengl/CPointCloudColoured.h>
@@ -258,3 +259,10 @@ void CPointCloudColoured::recolorizeByCoordinate(
 		this->setPointColor_fast(i, r, g, b);
 	}
 }
+
+void CPointCloudColoured::toYAMLMap(mrpt::containers::yaml& propertiesMap) const
+{
+	CRenderizable::toYAMLMap(propertiesMap);
+	propertiesMap["point_count"] = m_points.size();
+	propertiesMap["bounding_box"] = this->getBoundingBox().asString();
+}
